-- $Id: aragorn.lua 3307 2018-03-24 02:34:16Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2012-2018 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

local Quest = require 'engine.Quest'

newChat {
  id = 'welcome',
  text = [[Welcome back, @playername@.]],
  answers = {
    {
      "I have cleared the necromancer's lair, Your Highness.",
      cond = function(npc, player)
	return player:isQuestStatus('minas-anor-house', Quest.COMPLETED)
      end,
      jump = 'house-done',
    },
    {
      'Here is the information Turgon sent, Your Highness. #LIGHT_GREEN#*give the information to Turgon*#LAST#',
      cond = function(npc, player)
	return player:isQuestStatus('last-alliance', Quest.COMPLETED, 'gondolin') and not player:isQuestStatus('last-alliance', Quest.DONE)
      end,
      jump = 'alliance-done',
    },
    { 'Excuse me, Your Highness. [leave]' },
  },
}

newChat {
  id = 'give-house',
  text = [[Welcome and well met, @playername@.  Tales of your valor precede you.  In fact, I'm hoping I can prevail upon your skills to look into something for me.]],
  answers = {
    { 'What do you need, Your Highness?', jump='house-quest' },
  },
}

newChat {
  id = 'house-quest',
  text = [[It seems we have, or rather had, a necromancer living in a house on the edge of town.  Him you need not worry about; rumor has it he was slain by his own creations when he lost control of them.  But without him, those undead creatures are making increasingly brave forays out of his lair and attacking my people.  I need you to clean up the mess the fool left behind.]],
  answers = {
    {
      "Killing undead is actually something of a hobby of mine, if I do say so myself.  I'm in.",
      action = function(npc, player)
	player:grantQuest('minas-anor-house')
	util.place_quest_entrance('quest', 'house', 'ENTER_MINAS_ANOR_HOUSE_QUEST', true)
      end,
    },
    { 'Ew, undead.  They decompose all over you. Pass. [leave]' },
  },
}

newChat {
  id = 'house-done',
  text = [[Ah, that will be a relief to the city guard.  Perhaps we can get people to move back into that part of the city again.  Not that anyone is likely to touch the fool necromancer's lair anytime soon... in fact, I'll make you a deal:  clean out the necromancy gear and the corpse bits, and I'll place the house in your name.]],
  answers = {
    {
      'Thank you, Your Highness... I think.',
      action = function(npc, player)
	player:setQuestStatus('minas-anor-house', Quest.DONE)
	util.place_home('quest', 'house', 'PERMA_WALL', 'MINAS_ANOR_HOME')
      end,
    }
  }
}

newChat {
  id = 'give-alliance',
  text = [[Ah, @playername@, you were just in my thoughts.  I have a task that needs done, but it is a difficult one, #YELLOW##{italic}#(estimated difficulty level 80)#{normal}##LAST# so I will understand if you are not interested.]],
  answers = {
    { "That does sound dangerous.  I'll return when I am stronger, Your Highness. [leave]" },
    {
      "That doesn't sound like something I can't handle.  What is this task?",
      jump = 'alliance-quest',
    },
  },
}

newChat {
  id = 'alliance-quest',
  text = [[Our allies in Gondolin have proposed a quite audacious project:  with the combined knowledge and skills of their Thunderlords and ours, we should be able to create a void jumpgate connecting our two kingdoms.  The ability to coordinate our actions and reinforce each other's defenses will be invaluable against the armies of Morgoth.  Most of the preparations are completed; we need only the final linking information from Gondolin's Thunderlords.  I will send word to King Turgon to expect you; get the information from him and bring it back to us.  Have a care, though — a wing of Thunderlords under Trone have rebeled and broken away, and they oppose the creation of this gate.  They will likely try to stop you.]],
  answers = {
    {
      "I will return with the information, Your Highness.",
      action = function(npc, player)
	player:grantQuest('last-alliance')
      end,
    },
    { "Rebel Thunderlords?  Changed my mind, I'm out. [leave]" },
  },
}

newChat {
  id = 'alliance-done',
  text = [[Excellent! #LIGHT_GREEN#*He hands the information to an aide nearby, who rushes out of the throne room with it*#LAST#  My Thunderlords have been champing at the bit to complete our end of the jumpgate; from what they've told me, with that information they may well have it completed by the time you leave the castle.  You have done a great service to all the free lands of Arda, @playername@.  I hereby grant you free use of the jumpgate you helped create.  Oh, and the Thunderlords asked me to give you this as a show of their own gratitude.]],
  answers = {
    {
      'It was an honor, Your Highness.  A rather harrowing one, but still.',
      action = function(npc, player)
	-- The quest's on_status_change() will give us the Golden Horn of
	-- the Thunderlords.
	player:setQuestStatus('last-alliance', Quest.DONE)
	util.place_quest_entrance('quest', 'between-gate', 'MINAS_ANOR_VOID_JUMPGATE', true)
      end,
    }
  },
}

newChat {
  id = 'narsil1',
  text = [[Welcome, @playername@.  My sources tell me you have brought something of great interest to show me.]],
  answers = {
    { [[I have indeed, Your Highness. #LIGHT_GREEN#*show Narsil to Aragorn*#LAST#]], jump = 'narsil2' },
  },
}

newChat {
  id = 'narsil2',
  text = [[...Remarkable.  This is truly Narsil, the sword of my ancestors, broken so many years ago.  Where did you find­ no, that is not important.  Here, let me take it to the royal swordsmiths.  The sword that was broken shall be reforged at last.]],
  answers = {
    {
      [[#LIGHT_GREEN#*give Narsil to Aragorn*#LAST#]],
      action = function(npc, player)
	local o, item, inven_id = player:findInAllInventoriesBy('define_as', 'NARSIL')
	if o then
	  -- Sneaky Hack(TM), part 1:  Keep a copy of Narsil's found{} info
	  -- to put on Andúril.
	  player.tmp_narsil_found = o.found:clone()
	  player:removeObject(inven_id, item, true)
	  -- We do NOT call o:removed() here, as that would put Narsil back
	  -- in circulation, which would just be silly.
	  if game and game.hasEntity and game:hasEntity(o) then game:removeEntity(o) end
	end
      end,
      jump = 'narsil3'
    },
  },
}

newChat {
  id = 'narsil3',
  text = [[#LIGHT_GREEN#*Aragorn returns a few hours later.*#LAST#  Behold, Andúril, Flame of the West, the Sword That Was Broken and is now reforged.  Here, I offer it into your keeping for now, @playername@; my heart tells me that you have a momentous fate ahead of you, and you will have need of a sturdy weapon.]],
  answers = {
    {
      [[Thank you, Your Highness. #LIGHT_GREEN#*take Andúril*#LAST#]],
      action = function(npc, player)
	local o = game.zone:makeEntityByName(game.level, 'artifact', 'ANDURIL')
	if o then
	  o:identify('full', player)
	  -- Sneaky Hack(TM), part 2:  Munge Andúril's found information to
	  -- try to remember where we found Narsil.
	  o.found = player.tmp_narsil_found or { type='missing-narsil-info' }
	  player.tmp_narsil_found = nil
	  local ff = o.found
	  local pfx = '%s was reforged from the shards of Narsil, which '
	  if ff.type == 'birth' then
	    ff.type = 'custom'
	    ff.custom = pfx .. 'you began the game with (how?).'
	  elseif ff.type == 'floor' then
	    ff.type = 'custom_with_level'
	    ff.custom = pfx .. 'you found on the '..(ff.town_zone and 'ground' or 'floor')
	  elseif ff.type == 'mon_drop' then
	    ff.type = 'custom_with_level'
	    ff.custom = pfx .. 'you found in the remains of '..ff.mon_name
	  elseif ff.type == 'vault' then
	    ff.type = 'custom_with_level'
	    ff.custom = pfx .. 'you found in a vault'
	  elseif ff.type == 'rubble' then
	    ff.type = 'custom_with_level'
	    ff.custom = pfx .. 'you found buried under rubble'
	  elseif ff.type == 'chest' then
	    ff.type = 'custom_with_level'
	    ff.custom = pfx .. 'you found in a chest'
	  elseif ff.type == 'store_buy' then
	    ff.type = 'custom_with_level'
	    ff.custom = pfx .. 'you bought from the '..ff.store_name
	  elseif ff.type == 'obj_create' then
	    ff.type = 'custom_with_level'
	    ff.custom = pfx .. 'you created (how?)'
	  elseif ff.type == 'debug_dialog' then
	    ff.type = 'custom'
	    ff.custom = pfx .. 'you created from the debug dialog.'
	  elseif ff.type == 'custom' or ff.type == 'custom_with_level' then
	    -- Questionable Hack(TM):  Assume the custom text is roughly of
	    -- the form 'You magically conjured %s into existence', which
	    -- we convert to 'you magically conjured into existence'.
	    ff.custom = pfx .. ff.custom:gsub('You', 'you', 1):gsub(' %%s', '', 1)
	  else
	    ff.custom = '??? ' .. pfx .. ("you found somewhere strange (type='%s')"):format(tostring(ff.type or '???'))
	    ff.type = 'custom'
	  end
	  player:addOrDropObject(player.INVEN_INVEN, o, nil, nil, {vocal_drop=true})
	  player:sortInven()
	  game.logPlayer(player, 'You receive:  %s', o:getName())
	  player:setQuestStatus('narsil', engine.Quest.DONE)
	  world:gainAchievement('NARSIL', player)
	end
      end,
    },
  },
}

local q_complete = {
  { 'lothlorien-house', nil },
  { 'last-alliance', 'gondolin' },
}
local q_next = {
  { need='minas-anor-house', jump='give-house' },
  { done='minas-anor-house', need='last-alliance', jump='give-alliance' },
}

if game.player:findInAllInventoriesBy('define_as', 'NARSIL') then
  return 'narsil1'
end
for _, grp in ipairs(q_complete) do
  if player:isQuestStatus(grp[1], Quest.COMPLETED, grp[2]) then
    return 'welcome'
  end
end
for _, grp in ipairs(q_next) do
  if (not grp.done or player:isQuestStatus(grp.done, Quest.DONE)) and not player:hasQuest(grp.need) then
    return grp.jump
  end
end
return 'welcome'
