-- $Id: beastmaster-shanty.lua 3333 2018-04-20 00:32:38Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2012-2018 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

store_util.set_prices(npc.owner, player, { research={1600, 1500, 1400} })

local function lore_gain_fmt(player)
  local s = player.skills.MONSTER_LORE
  if s.mod == 0 then
    local m = 800/player.skill_step
    return ('#LIGHT_BLUE#(+%g Monster-lore skill, %g multiplier)#LAST#'):format(m, m)
  else
    return ('#LIGHT_BLUE#(+%g Monster-lore skill)#LAST#'):format(s.mod/player.skill_step)
  end
end

local function has_beast_quest(player, mode)
  for k, q in pairs(player.quests) do
    if k:find('^beastmaster-') then
      local ok = false
      if mode == 'pending' then
	-- "Pending" quest:  not completed and haven't found corpse
	ok = not q:isEnded() and not q:has_target_corpse(player)
      elseif mode == 'found' then
	-- "Found" quest:  not completed and have found corpse
	ok = not q:isEnded() and q:has_target_corpse(player)
      elseif mode == 'active' then
	-- "Active" quest:  not completed, don't care whether found corpse
	ok = not q:isEnded()
      else
	-- Anything else means any quest, completed or not.
	ok = true
      end
      if ok then return q end
    end
  end
  return nil
end

local function research_monster(npc, player)
  local Research = require 'mod.dialogs.BeastmasterResearch'
  game:registerDialog(Research.new(player, prices.research))
end

local function pick_bounty_monster(npc, player, dlg)
  -- cf. lua_mon_hook_bounty() in src/lua_bind.c
  local function filter_cb(e)
    return not e.unique and
	   e.drop_corpse and e.drop_corpse.corpse and
	   not e:has('SPECIAL_GENE') and not e:has('NEVER_GENE') and
	   not e:has('PET') and not e:has('FRIENDLY') and
	   not e:has('MULTIPLY') and
	   -- TODO No JokeAngband monsters, if we ever add those.
	   not e:has('GOOD') and
	   -- New for us:  don't return a monster we've already had a
	   -- bounty quest for, as that would confuse the quest
	   -- infrastructure.
	   not player:hasQuest('beastmaster-'..e.define_as)
  end
  -- This roughly duplicates the pre-resolving part of Zone:makeEntity()
  -- with prob_filter=true.
  local fake_level = {
    level = 3 + math.floor((player.level*3)/2),
    getEntitiesList = function(x) return {} end,
  }
  local list = game.zone:computeRarities('actor', game.zone.npc_list, fake_level, filter_cb)
  for _ = 1, 50 do
    local e = game.zone:pickEntity(list)
    if e then
      -- Deep wizardry:  we reach ahead into the next chat object and
      -- replace its text and answer with ours.
      local chat = dlg.chat:get('quest2')
      local name = e:getName{indef_art=true}
      chat.text = [[Bonzer!  Right, let's see what we've got on the wish list, then... ooh, `ow `bout you bring us ]]..name..[[?]]
      chat.answers = {
	{
	  [[I'll be back with ]]..name..[[ corpse.]],
	  action = function(npc_, player_)
	    player_:grantQuest('beastmaster', {target_tag=e.define_as})
	  end,
	}
      }
      return
    end
  end
end

local function return_corpse_answer(player)
  local q = has_beast_quest(player, 'found')
  if q then
    return ('I have brought back %s corpse.'):format(q:target_name())
  else
    return [[#LIGHT_RED#If you are seeing this, it is a bug#LAST#]]
  end
end

newChat {
  id = 'welcome',
  text = [[#LIGHT_GREEN#*A Human wearing an odd-looking hat with a rather intimidating machete hanging from his belt looks up from his work as you enter.*#LAST#
G'day, mate!  Welcome to the Beastmaster Shanty, where we're all about our furry and scaly and feathery friends.  You name a beastie, we can teach you all about it.  Or if you #{italic}#really#{normal}# hear the call o' the wild, we can teach you more general lore about dealin' with all beasties.]],
  answers = {
    {
      'I need information about a particular monster.' .. store_util.cost_sfx('research'),
      cond = store_util.can_afford('research'),
      action = research_monster,
    },
    {
      [["More general lore"?  You mean the Monster-lore skill?]],
      jump = 'quest1',
    },
    { 'Oh, sorry, I was looking for the armor store. [leave]' },
  },
}

newChat {
  id = 'quest1',
  text = [[Yep, we call it our Mutual Work Study program.  We pick a beastie we need to do more research on, you bring us back the body o' one, we study it, and you gain some lore ]]..lore_gain_fmt(player)..[[.  And if you don't know the Corpse-preservation skill, heck, we'll teach you that too #LIGHT_BLUE#(+0.8 Corpse-preservation skill, 0.8 multiplier)#LAST#.  Got to know that sort o' thing when you're survivin' out in the wild, right?  So, sound good?]],
  answers = {
    { "I'm in.", action=pick_bounty_monster, jump='quest2' },
    { 'Nah, sounds like too much hassle. [leave]' },
  }
}

newChat {
  id = 'quest2',
  text = [[...Well, bugger.  The quest must be broken; this text ought to have been replaced by now.  Maybe we couldn't find a bounty monster?]],
  answers = {
    { '#{italic}#*blink*#{normal}# O-kay, then... [leave]' },
  }
}

newChat {
  id = 'welcome2',
  text = [[Ah, g'day, @playername@.  `Ow goes the explorin'?]],
  answers = {
    {
      return_corpse_answer(player),
      cond = function(npc, player) return has_beast_quest(player, 'found') end,
      jump = 'quest3',
    },
    {
      'I need information about a particular monster.' .. store_util.cost_sfx('research'),
      cond = store_util.can_afford('research'),
      action = research_monster,
    },
    {
      [[I could use some more Monster-lore skill.  Need any more corpses?]],
      cond = function(npc, player)
	return not has_beast_quest(player, 'active')
      end,
      action = pick_bounty_monster,
      jump = 'quest2'
    },
    { "Pretty good.  I'ma get back to that exploring now, in fact. [leave]" },
  },
}

newChat {
  id = 'quest3',
  text = [[Bonzer!  Let's `ave a butcher's, then.  Cor, she's a beaut, ain't she?  Right, let's haul it back to the taxidermy lab and see what we can learn from it.]],
  answers = {
    {
      [[Right behind you.]],
      action = function(npc, player)
	local q = has_beast_quest(player, 'found')
	if q then
	  q:remove_target_corpse(player)
	  -- This will adjust our Monster-lore and/or Corpse-preservation
	  -- skills.
	  q:setStatus(q.DONE, nil, player)
	end
      end
    }
  },
}

if has_beast_quest(player, 'any') then
  -- Player has any active or completed bounty quests.
  return 'welcome2'
end
return 'welcome'
