-- $Id: lothlorien-seat-of-ruling.lua 3333 2018-04-20 00:32:38Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2018 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

local Quest = require 'engine.Quest'

newChat {
  id = 'welcome',
  text = [[Welcome, @playername@.]],
  answers = {
    {
      "I have cleared the wolves' den, Milord.",
      cond = function(npc, player)
	return player:isQuestStatus('lothlorien-house', Quest.COMPLETED)
      end,
      jump = 'house-done',
    },
    {
      "I have slain the spiders, Milord.",
      cond = function(npc, player)
	return player:isQuestStatus('lothlorien-spiders', Quest.COMPLETED)
      end,
      jump = 'spiders-done',
    },
    {
      "I have purified the waters of Nimrodel, Milord.",
      cond = function(npc, player)
	return player:isQuestStatus('lothlorien-poisoned-water', Quest.COMPLETED)
      end,
      jump = 'water-done',
    },
    { 'Excuse me, Milord. [leave]' },
  }
}

newChat {
  id = 'give-house',
  text = [[Welcome, @playername@.  I hear you have made something of a name for yourself in the human town of Bree.  Perhaps you can help us with a task.]],
  answers = {
    { 'What do you need?', jump='house-quest' },
  },
}

newChat {
  id = 'house-quest',
  text = [[There is an old abandoned warehouse east of here that has been overrun by a pack of wolves.  They have been harrassing my people and being a general nuisance.  I need them cleared out.]],
  answers = {
    {
      'I can do that, no problem.',
      action = function(npc, player)
	player:grantQuest('lothlorien-house')
	util.place_quest_entrance('quest', 'house', 'ENTER_LOTHLORIEN_HOUSE_QUEST', true)
      end,
    },
    { 'Bleh, wolves means fleas.  No thanks. [leave]' },
  }
}

newChat {
  id = 'house-done',
  text = [[So I have heard.  Well done, @playername@.  So far as I know, no one has any claim on that warehouse any longer; I will place it in your name, to use as you see fit.]],
  answers = {
    {
      'Thank you, Milord.',
      action = function(npc, player)
	player:setQuestStatus('lothlorien-house', Quest.DONE)
	util.place_home('quest', 'house', 'PERMA_WALL', 'LOTHLORIEN_HOME')
      end,
    },
  }
}

newChat {
  id = 'give-spiders',
  text = [[Welcome, @playername@.  Since you fared so well with the wolves, perhaps you can assist with another matter.]],
  answers = {
    { 'What do you need?', jump='spiders-quest' },
  },
}

newChat {
  id = 'spiders-quest',
  text = [[My scouts report that spiders are massing from Mirkwood and blocking the path through the forest.  Thranduil's folk have always been able to hold them off in the past, but it seems they have been overwhelmed.  Your task will be to drive them away.  Have a care, though; many traps have been laid by their webs, and their venom is dangerous indeed.]],
  answers = {
    {
      'I think I can handle some spiders.',
      action = function(npc, player)
	player:grantQuest('lothlorien-spiders')
	util.place_quest_entrance('quest', 'spiders', 'ENTER_LOTHLORIEN_SPIDERS_QUEST', true)
      end,
    },
    { 'Yyyeah, did I mention my chronic arachnophobia? [leave]' },
  },
}

newChat {
  id = 'spiders-done',
  text = [[Yes, my scouts tell me the paths are blessedly clear of spiders again.  Thranduil's archers send their thanks, along with a token of their esteem.]],
  answers = {
    {
      'Then relay to them my gratitude, Milord.',
      action = function(npc, player)
	-- The quest's on_status_change() will give us the Potion of
	-- Augmentation.
	player:setQuestStatus('lothlorien-spiders', Quest.DONE)
      end,
    },
  },
}

newChat {
  id = 'give-water',
  text = [[Welcome, @playername@.  After your fight with the spiders, I hesitate to impose upon you again, but this matter may well require your unique skills.]],
  answers = {
    { 'What do you need?', jump='water-quest' },
  },
}

newChat {
  id = 'water-quest',
  text = [[Ill news reaches my ears.  The trees along the banks of Nimrodel are withering away, and we fear the blight may spread to consume the whole forest.  We suspect some sort of poison has entered the waters.  Your task is to follow Celebrant and Nimrodel westward until you discover the source of the poison.  Once you have eradicated it, drop these potions into the tainted water to purify it.]],
  answers = {
    {
      [[Blech.  Dirty job, but I guess someone's gotta do it.]],
      action = function(npc, player)
	-- The quest's on_grant() method will give the player the Potions
	-- of Water Curing.
	player:grantQuest('lothlorien-poisoned-water')
      end,
    },
    { "Y'know, I just remembered, I can't swim. [leavel]" },
  },
}

newChat {
  id = 'water-done',
  text = [[Yes, the ferriers have already begun traversing the Nimrodel again, now that the waters are clear.  They have sent an enchanted item from their stores as thanks to you.]],
  answers = {
    {
      'Then relay to them my gratitude, Milord.',
      action = function(npc, player)
	-- The quest's on_status_change() will give us reward item.
	player:setQuestStatus('lothlorien-poisoned-water', Quest.DONE)
      end,
    },
  },
}

local q_complete = {
  'lothlorien-house', 'lothlorien-spiders', 'lothlorien-poisoned-water',
}
local q_next = {
  { need='lothlorien-house', jump='give-house' },
  { done='lothlorien-house', need='lothlorien-spiders', jump='give-spiders' },
  { done='lothlorien-spiders', need='lothlorien-poisoned-water', jump='give-water' },
}

for _, tag in ipairs(q_complete) do
  if player:isQuestStatus(tag, Quest.COMPLETED) then return 'welcome' end
end
for _, grp in ipairs(q_next) do
  if (not grp.done or player:isQuestStatus(grp.done, Quest.DONE)) and not player:hasQuest(grp.need) then
    return grp.jump
  end
end
return 'welcome'
