-- $Id: beastmaster.lua 3333 2018-04-20 00:32:38Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2012-2018 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

id = 'beastmaster-' .. (target_tag or 'DUMMY')
name = 'Beastmaster Bounty - ' .. (target_tag and game.zone.npc_list[target_tag] and game.zone.npc_list[target_tag]:getName() or '???')

desc = function(self, who)
  local name = self:target_name()

  if self:isStatus(self.DONE) then
    return ([[You collected the corpse of %s for the Beastmaster.  In return, %s.]]):format(name, self.reward_desc)
  elseif who:findInInventoryBy(who:getInven('INVEN'), 'mon_id', self.target_tag)  then
    return ([[You have obtained the corpse of %s that you agreed to collect for the Beastmaster.  You should take it to the Beastmaster Shanty.]]):format(name)
  else
    return ([[You have agreed to collect the corpse of %s for the Beastmaster.]]):format(name)
  end
end

on_status_change = function(self, who, status, sub)
  if status == self.DONE then
    local s_ml, s_cp = who.skills.MONSTER_LORE, who.skills.CORPSE_PRESERVATION
    if s_ml.mod == 0 then
      s_ml.mod = 800
      who:setSkillValue('MONSTER_LORE', 800)
      self.reward_desc = 'he taught you the Monster-lore skill'
      if s_cp.mod == 0 then
	self.reward_desc = 'he taught you the Monster-lore and Corpse-preservation skills'
      end
      game.logPlayer(who, '#LIGHT_BLUE#You have learned the Monster-lore skill.')
    else
      who:adjSkillValue('MONSTER_LORE', s_ml.mod)
      self.reward_desc = ('he increased your Monster-lore skill by %g.'):format(s_ml.mod/who.skill_step)
      if s_cp.mod == 0 then
	self.reward_desc = self.reward_desc .. ' and taught you the Corpse-preservation skill'
      end
      game.logPlayer(who, '#LIGHT_BLUE#Your Monster-lore skill is increased.')
    end
    if s_cp.mod == 0 then
      s_cp.mod = 800
      who:setSkillValue('CORPSE_PRESERVATION', 800)
      game.logPlayer(who, '#LIGHT_BLUE#You have learned the Corpse-preservation skill.')
    end
  elseif status == self.COMPLETED and sub == 'found' then
    local Dialog = require 'engine.ui.Dialog'
    Dialog:simpleLongPopup(self.name, [[You have collected the corpse requested by the Beastmaster.  You should take it back to the Beastmaster Shanty.]], 600)
  end
end

target_name = function(self, short)
  local a = game.zone.npc_list[self.target_tag]
  if not a then return '???' end
  return a:getName{indef_art=not short}
end

has_target_corpse = function(self, who)
  local inven = who:getInven('INVEN')
  return who:findInInventoryBy(inven, 'mon_id', self.target_tag)
end

remove_target_corpse = function(self, who)
  local inven = who:getInven('INVEN')
  local o, item = who:findInInventoryBy(inven, 'mon_id', self.target_tag)
  if o and item then
    who:removeObject(inven, item, 1)
    return true
  else
    return false
  end
end

callbacks = {
  on_pickup_floor = function(who, q, o, n)
    if o.type == 'misc' and o.subtype == 'corpse' and o.mon_id == q.target_tag then
      q:setStatus(q.COMPLETED, 'found', who)
    end
  end,
}
