-- $Id: bree-trolls.lua 3522 2018-12-15 20:41:49Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2012-2018 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

id = 'bree-trolls'
name = 'The Trolls Glade'
danger_level = 30	-- cf. src/tables.c line 3821
desc = function(self, who)
  if self:isEnded() then
    return [[You have slain Tom the Stone Troll and taken the Elven sword Glamdring from him.  Mayor Will Whitfoot thanks you and has allowed you to keep Glamdring for your own use.]]
  elseif self:isCompleted() then
    return [[You have slain Tom the Stone Troll and taken the Elven sword Glamdring from him.  You should take it back to the Mayor's office and show it to the mayor.]]
  else
    return [[You must slay the forest trolls camped in the forest southeast of Bree.  Reports say their camp can only be found at night.]]
  end
end

on_status_change = function(self, who, status, sub)
  -- Only interested in kills in the quest zone.
  if game.zone.short_name ~= 'quest-bree-trolls' then return end
  if target.define_as == 'TOM_THE_STONE_TROLL' then
    -- Killed Tom; we're done.
    game.log("#YELLOW#Without Tom, the trolls won't be able to do much.#LAST#")
    local spot = game.level:pickSpot{ type='trolls-quest', subtype='exit' }
    local Map = require 'mod.class.Map'
    game.level.map(spot.x, spot.y, Map.TERRAIN, game.zone.grid_list.EXIT_BREE_TROLL_QUEST)
    game.player:setQuestStatus('bree-trolls', engine.Quest.COMPLETED)
  elseif not self.done_ambush then
    -- Killed anyone else; trigger the ambush.
    self.done_ambush = true
    local Map = require 'mod.class.Map'
    local _, spots = game.level:pickSpot{ type='trolls-quest', subtype='ambush' }
    for _, spot in ipairs(spots) do
      game.level.map(spot.x, spot.y, Map.TERRAIN, game.zone.grid_list.GRASS)
      local tag = rng.chance(2) and 'FOREST_TROLL' or 'STONE_TROLL'
      local m = game.zone:makeEntityByName(game.level, 'actor', tag, false)
      if m then game.level.map(spot.x, spot.y, Map.ACTOR, m) end
    end
    local Dialog = require 'engine.ui.Dialog'
    local name = target:getName{def_art=true, check_seen=true}
    local fmt = [[As %s falls, you hear a hue and cry, and several more trolls emerge from the trees.  This was an ambush!]]
    Dialog:simpleLongPopup('The Trolls Glade', fmt:format(name), 600)
  end
end
