-- $Id: relic.lua 3324 2018-04-14 23:42:31Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2012-2018 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

id = 'relic'
name = 'Lost Relic'

add_hooks = function()
  class:bindHook('Actor:levelupPost', function(self, data)
    local a = data.actor
    if a.player and a.descriptor and a.descriptor.god ~= 'Nobody' and not a:has('ASTRAL') then
      local q = a:hasQuest('relic')
      if (not q or (not q:isEnded() and q:num_pending() == 0)) and (rng.percent(21) or a.force_relic_quest) then
	a.force_relic_quest = nil
	local Birther = require 'engine.Birther'
	local god_d = Birther:getBirthDescriptor('god', a.descriptor.god)
	local o = game.zone.object_list[god_d.quest.relic]
	local npc = { name=god_d.name, chat_display_entity=o }
	local Chat = require('engine.Chat')
	local chat = Chat.new('quest-relic', npc, a)
	ui_util.invoke_chat(chat)
      end
    end
  end)

  class:bindHook('CharacterSheet:dumpLine', function(self, data)
    local q = game.player:hasQuest('relic')
    if not q then return end
    local n = q:num_done()
    local nf = n == 0 and 'none' or tostring(n)
    local line
    if n == 5 then
      line = 'You found all of the relic pieces and pleased your god.'
    elseif q:isStatus(engine.Quest.FAILED) then
      line = ('You found %s of the relic pieces and failed in your quest.'):format(nf)
    else
      line = ('You found %s of the relic pieces.'):format(nf)
    end
    table.insert(data.lines, line)
  end)
end

num_done = function(self)
  local n = 0
  for i = 1, 5 do if self:isCompleted(i) then n = i end end
  return n
end
num_pending = function(self)
  for i = 1, 5 do
    if self:isStatus(engine.Quest.PENDING, i) then return i end
  end
  return 0
end

desc = function(self, who)
  local god = who.descriptor.god

  if self:isStatus(engine.Quest.DONE) then
    return ([[You have successfully retrieved all five parts of %s's holy relic.]]):format(god)
  end

  local n_done = self:num_done()
  local n_pend = self:num_pending()
  local short_god = string.match(god, '^[^%s]+')
  local god_d = require('engine.Birther'):getBirthDescriptor('god', god)
  -- Slight Hack(TM):  All the god descriptions begin with "He/She is...".
  local hir = god_d.desc:find('^She') and 'her' or 'his'
  local ret = [[You have been commanded by %s to retrieve the pieces of %s lost relic.]]
  ret = ret:format(god, hir)
  if n_done > 0 then
    local bits = n_done == 1 and 'one fragment' or ('%d fragments'):format(n_done)
    ret = ret .. ([[  You have retrieved %s so far; ]]):format(bits)
    if n_pend > 0 then
      local nth = { 'first', 'second', 'third', 'fourth', 'final' }
      local loc = self:temple_loc_desc(who)
      ret = ret .. ([[the %s piece is in a lost temple %s.]]):format(nth[n_pend], loc)
    else
      ret = ret .. ([[%s has not yet told you where any other pieces might be found.]]):format(short_god)
    end
  elseif n_pend == 1 then
    ret = ret .. '  The first piece is in a lost temple ' .. self:temple_loc_desc(who) .. '.'
  end
  return ret
end

temple_loc_desc = function(self, who)
  local town1, town2, t1_loc, t2_loc
  if who:isFollowing('Melkor') then
    town1, town2 = 'the Pits of Angband', 'the Land of Mordor'
    t1_loc = game.state:getWorldMapZoneCoords('angband')
    t2_loc = game.state:getWorldMapZoneCoords('mordor')
  else
    town1, town2 = 'Bree', 'Minas Anor'
    t1_loc = game.state:getWorldMapZoneCoords('town-bree')
    t2_loc = game.state:getWorldMapZoneCoords('town-minas-anor')
  end

  local function landmark_ref(town_xy, quest_xy, town_name)
    local dx, dy = quest_xy.x - town_xy.x, quest_xy.y - town_xy.y
    local d = math.max(math.abs(dx), math.abs(dy))
    if d < 3 then return 'very close to ' .. town_name end
    local dir = (math.abs(dy) < 3 and '' or dy > 0 and 'south' or 'north') ..
		(math.abs(dx) < 3 and '' or dx > 0 and 'east' or 'west')
    local dist = d >= 41 and 'a very long way' or
		 d >= 25 and 'a long way' or
		 d >= 8  and 'quite some way' or 'not very far'
    return ([[%s to the %s of %s]]):format(dist, dir, town_name)
  end

  return landmark_ref(t1_loc, self.zone_loc, town1) .. ' and ' ..
	 landmark_ref(t2_loc, self.zone_loc, town2)
end

place_zone = function(self)
  if self.zone_loc then
    -- Forget the old quest location.
    game.state:setWildernessQuest(self.zone_loc.x, self.zone_loc.y, nil)
    -- Clear recall depth for the removed dungeon.
    game.state.zone_recall_depth['lost-temple'] = nil
    game.state.zone_info['lost-temple'] = nil
    if game.state.recall_dungeon.zone == 'lost-temple' then
      game.state.recall_dungeon = nil
    end
  end

  local bad_types = {
    EDGE=true, DEEP_WATER=true, TREES=true, SHALLOW_LAVA=true,
    DEEP_LAVA=true, MOUNTAIN=true,
  }
  local wh = game.state:getWorldMapSize()
  local qx, qy
  for _ = 1, 1000 do
    local x, y = rng.range(1, wh.w - 3), rng.range(1, wh.h - 3)
    local g = game.state:getWorldGrid(x, y)
    if not bad_types[g.terrain_type] and not g.change_zone then
      qx, qy = x, y
      break
    end
  end
  -- Emergency fallback quest location is near Bree.
  self.zone_loc = qx and qy and { x=qx, y=qy } or { x=32, y=19 }
  -- Quest dungeon top level is 2/3 player lever at grant time.
  self.top_level = math.floor((game.player.level*2)/3)
  -- Change from T2!  Predetermine the level that has the relic piece on
  -- it.
  self.relic_level = rng.range(1, 5)
  print(('[RELIC] quest dungeon at %d,%d; relic on level %d'):format(self.zone_loc.x, self.zone_loc.y, self.relic_level))
  -- Register the new quest location.
  game.state:setWildernessQuest(self.zone_loc.x, self.zone_loc.y, 'relic')
end

on_grant = function(self, who)
  -- So that num_pending() will work for the first piece.
  self.objectives[1] = engine.Quest.PENDING
  self:place_zone()
end

on_status_change = function(self, who, status, sub)
  if sub then
    if self.objectives[sub] == engine.Quest.PENDING then
      -- Just granted next relic piece subquest; place the quest dungeon.
      self:place_zone()
    elseif self.objectives[sub] == engine.Quest.COMPLETED then
      -- Completed subquest by picking up the relic piece.
      local Birther = require 'engine.Birther'
      local god_d = Birther:getBirthDescriptor('god', who.descriptor.god)
      local o = game.zone.object_list[god_d.quest.relic]
      local npc = { name=god_d.name, chat_display_entity=o }
      local Chat = require 'engine.Chat'
      local chat = Chat.new('quest-relic-find', npc, who)
      ui_util.invoke_chat(chat)
    end
  end
end

worldmap_change_grid = function(self, x, y, orig_grid)
  if self:num_pending() == 0 then return nil end
  return orig_grid.temple_entrance or 'WORLD_LOST_TEMPLE_GRASS'
end
