-- $Id: thrain.lua 3333 2018-04-20 00:32:38Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2012-2018 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

add_hooks = function()
  class:bindHook('Game:levelFeeling', function(self, data)
    if data.zone.short_name ~= 'dol-guldur' then return end
    local qt = game.player:hasQuest('thrain')
    if qt and (qt:isEnded() or qt:isCompleted()) then
      -- Already rescued Thrain.
      return
    end
    -- Slight Hack(TM):  Thrain quest level info is stored in the
    -- Necromancer quest data.
    local qn = game.player:hasQuest('necromancer')
    if qn and data.lev == qn.thrain_level then
      game.log("#YELLOW#You hear someone shouting under the strain.#LAST#")
      if not qt then
	local lev = data.zone:level_adjust_level(data.level, 'zone')
	game.player:grantQuest('thrain', { thrain_lev_desc=lev })
      end
    end
  end)

  class:bindHook('MapGenerator:T2DungeonLevel:firstRoom', function(self, data)
    -- Not if someone else has already created the first room.
    if data.done then return end
    -- Not if we've already completed the quest.
    local qt = game.player:hasQuest('thrain')
    if qt and (qt:isEnded() or qt:isCompleted()) then
      return
    end
    -- Only in Dol Guldur
    if data.gen.zone.short_name ~= 'dol-guldur' then return end
    -- Slight Hack(TM):  Thrain quest level info is stored in the
    -- Necromancer quest data.
    local qn = game.player:hasQuest('necromancer')
    if qn and data.lev == qn.thrain_level then
      -- Build the quest room.  This is loosely based on T4's 'lesser_vault'
      -- room.
      -- TODO Maybe consolidate this and the equivalent code in the
      -- princess quest into a common method in util{}.
      local Map = require 'mod.class.Map'
      local Static = require 'mod.class.generator.map.Static'
      local gen_data = table.clone(data.gen.data)
      gen_data.map = 'quests/thrain'

      -- Questionable Hack(TM):  Create a new map to load the quest room
      -- into, and wedge it in everywhere the generator will expect the level
      -- map to be.
      local room_map = Map.new(50, 50)
      local orig_gen_level_map = data.gen.level.map
      data.gen.level.map = room_map
      local orig_game_level = game.level
      game.level = data.gen.level

      -- Create a new Static map generator and load our room map.
      local room_gen = Static.new(data.gen.zone, room_map, data.gen.level, gen_data)
      local _, _, _, _, room_spots = room_gen:generate(data.lev, data.old_lev)

      -- Questionable Hack(TM), redux:  Put back the actual level map.
      game.level = orig_game_level
      data.gen.level.map = orig_gen_level_map

      -- Now that we have the room size, try to allocate it on the map.
      local width, height = room_map.w, room_map.h
      local ctr = data.gen:roomAlloc(width+2, height+2, false, data.gx, data.gy)
      if not ctr then
	error('[QUEST] ??? Could not roomAlloc() Thrain quest room')
	return
      end

      local y1 = ctr.y - math.floor(height/2)
      local x1 = ctr.x - math.floor(width/2)
      local y2 = y1 + height - 1
      local x2 = x1 + width - 1

      -- First build an ordinary rectangular room.  [We mostly duplicate
      -- data/rooms/basic.lua here.]
      --   - Floor
      for y = y1, y2 do
	for x = x1, x2 do
	  data.gen.map(x, y, Map.TERRAIN, data.gen:resolve('floor', nil, false, data.gen.cur_lev))
	  data.gen.map.lites(x, y, true)
	  data.gen.map.attrs(x, y, 'room', true)
	end
      end
      --   - Top and bottom walls
      for x = x1 - 1, x2 + 1 do
	data.gen.map(x, y1 - 1, Map.TERRAIN, data.gen:resolve('outer_wall', nil, false, data.gen.cur_lev))
	data.gen.map.lites(x, y1 - 1, true)
	data.gen.map.attrs(x, y1 - 1, 'room', true)
	data.gen.map(x, y2 + 1, Map.TERRAIN, data.gen:resolve('outer_wall', nil, false, data.gen.cur_lev))
	data.gen.map.lites(x, y2 + 1, true)
	data.gen.map.attrs(x, y2 + 1, 'room', true)
      end
      --   - Left and right walls
      for y = y1 - 1, y2 + 1 do
	data.gen.map(x1 - 1, y, Map.TERRAIN, data.gen:resolve('outer_wall', nil, false, data.gen.cur_lev))
	data.gen.map.lites(x1 - 1, y, true)
	data.gen.map.attrs(x1 - 1, y, 'room', true)
	data.gen.map(x2 + 1, y, Map.TERRAIN, data.gen:resolve('outer_wall', nil, false, data.gen.cur_lev))
	data.gen.map.lites(x2 + 1, y, true)
	data.gen.map.attrs(x2 + 1, y, 'room', true)
      end

      -- Then overlay the quest room map on top of the room we just built.
      data.gen.map:overlay(room_map, x1, y1)

      -- Pass the spots defined by the map back to the caller.
      for _, spot in ipairs(room_spots) do
	spot.x = spot.x + x1
	spot.y = spot.y + y1
      end
      table.append(data.spots, room_spots)

      -- Tell our hook caller that we succeeded.
      data.done = true
    end
  end)
end

id = 'thrain'
name = 'Prisoner of Dol Guldur'
damger_level = 60
desc = function(self, who)
  if self:isEnded() then
    return [[You found Thráin, the King Under the Mountain being held captive on level ]]..self.thrain_lev_desc..[[ of Dol Guldur.  He did not survive his torture, but before he died he told you that the Necromancer is in fact Sauron.]]
  else
    return [[You have heard cries of distress on level ]]..self.thrain_lev_desc..[[ of Dol Guldur.  You suspect there is a prisoner being held there and tortured.]]
  end
end

unhidden = false
unhide_room = function(self)
  if self.unhidden then return end

  local _, hidden = game.level:pickSpot{ type='thrain-quest', subtype='hidden' }
  hidden = table.clone(hidden, true)
  local thrain = game.level:pickSpot{ type='thrain-quest', subtype='thrain' }
  hidden[#hidden+1] = thrain

  local Map = require 'mod.class.Map'
  for _, spot in ipairs(hidden) do
    local g = game.level.map(spot.x, spot.y, Map.TERRAIN)
    g = g:clone()
    g.replace_display = nil
    g.on_move = nil
    g.block_sight = false
    game.level.map(spot.x, spot.y, Map.TERRAIN, g)
  end
  self.unhidden = true
  game.level.map.changed = true
  game.level.map:redisplay()
  game.log('#YELLOW#The magic hiding the room dissipates.')
end

guards = {
  DWAR_DOG_LORD_OF_WAW	= true,
  HOARMURATH_OF_DIR	= true,
}
guards_killed = {}

on_quest_mon_die = function(self, who, src)
  local tag = who.define_as
  if who.quest_mon and self.guards[tag] then
    local all = true
    self.guards_killed[tag] = true
    for t, _ in pairs(self.guards) do all = all and self.guards_killed[t] end
    if all then
      -- Killed both of them.
      self:unhide_room()
      local Map = require 'mod.class.Map'
      local Chat = require 'engine.Chat'
      local spot = game.level:pickSpot{ type='thrain-quest', subtype='thrain' }
      local npc = game.level.map(spot.x, spot.y, Map.ACTOR)
      -- In case Thrain has been killed somehow.
      npc = npc or ui_util.chat_npc('THRAIN_THE_KING_UNDER_THE_MOUNTAIN')
      local chat = Chat.new('quest-thrain-finish', npc, game.player)
      ui_util.invoke_chat(chat)
    end
  end
end
