-- $Id: crypt.lua 2835 2017-04-11 00:13:59Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2016-2017 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

-- Roughly mimics build_type12() in src/generate.c, to build a crypt room.
newRoom {
  id = 'crypt',
  min_depth = 10,	-- cf. roomdep[] in src/generate.c
  wgt_unusual = 10,	-- cf. src/generate.c lines 6973-6974
  build = function(self, gen, gx, gy)
    -- Random metric.
    local h1, h2 = rng.range(-15, 16), rng.range(1, 16)
    local h3, h4 = rng.range(1, 32), rng.range(-15, 16)
    -- Occasional light.
    local lit = rng.range(1, gen.dun_lev) <= 5

    local rad = rng.range(1, 9)
    local ctr = gen:roomAlloc(rad*2 + 3, rad*2 + 3, false, gx, gy)
    if not ctr then return false end

    for x = ctr.x - rad, ctr.x + rad do
      for y = ctr.y - rad, ctr.y + rad do
	-- Clear room flag, for use with :add_outer_wall()
	gen.map:unattr(x, y, 'room')
	-- Floor or wall, depending on distance from center.
	if gen:dist2(ctr.x, ctr.y, x, y, h1, h2, h3, h4) <= rad - 1 then
	  gen.map(x, y, Map.TERRAIN, gen:resolve('floor', nil, false, gen.cur_lev))
	elseif core.fov.distance(ctr.x, ctr.y, x, y) < 3 then
	  gen.map(x, y, Map.TERRAIN, gen:resolve('floor', nil, false, gen.cur_lev))
	else
	  -- :add_outer_wall() looks for WALL_XTRA's .xtra_hack
	  gen.map(x, y, Map.TERRAIN, gen.zone.grid_list.WALL_XTRA)
	end
      end
    end
    -- Outer walls
    gen:build_rectangle(ctr.x - rad, ctr.y - rad, ctr.x + rad, ctr.y + rad, 'outer_wall')

    -- Find visible outer walls and set them to outer_wall.
    gen:add_outer_wall(ctr.x, ctr.y, lit, ctr.x - rad - 1, ctr.y - rad - 1, ctr.x + rad + 1, ctr.y + rad + 1)

    -- Check to see if there is room for an inner vault.
    local empty = true
    for x = ctr.x - 2, ctr.x + 2 do
      for y = ctr.y - 2, ctr.y + 2 do
	empty = empty and gen:get_is_floor(x, y)
      end
    end

    if empty and rng.chance(2) then
      -- Build the vault.
      self:build_small_room(gen, ctr)
      -- Place treasure in the vault.
      gen:place_object(ctr.x, ctr.y)
      -- Guard the treasure.
      gen:place_vault_mon(ctr.x, ctr.y, rng.range(3, 4))
      -- And traps.
      gen:place_vault_traps(ctr.x, ctr.y, 4, 4, rng.range(2, 4))
    end
  end,
  build_small_room = function(self, gen, ctr)
    gen:build_rectangle(ctr.x - 1, ctr.y - 1, ctr.x + 1, ctr.y + 1, 'inner_wall')
    gen.map(ctr.x, ctr.y, Map.TERRAIN, gen:resolve('floor', nil, false, gen.cur_lev))
    -- Place a secret door on one side.
    local side = rng.range(0, 3)
    local dx = side >= 2 and 0 or side*2 - 1
    local dy = side < 2 and 0 or side*2 - 5
    gen:place_secret_door(ctr.x + dx, ctr.y + dy)
    return true
  end,
}
