-- $Id: zone.lua 3850 2020-04-05 20:13:34Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2012-2020 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

-- Zone file for Illusory Castle.

return {
  name = 'Illusory Castle',
  display_abbr = 'Ill',
  level_scheme = 'fixed',
  max_level = 18,
  level_range = { 35, 52 },
  width = 198,
  height = 66,
  effects = { 'EFF_ZONE_ILLUSORY_CASTLE' },

  generator = {
    map = {
      class = 'mod.class.generator.map.T2DungeonLevel',
      floor = { weighted = { FLOOR=98, GLASS_WALL=2 } },
      fill = { weighted = { WALL=50, ILLUSION_WALL=50 } },
      outer_wall = 'WALL',
      inner_wall = 'WALL',
      fill_step = 1,
      up = 'UP',
      down = 'DOWN',
      flags = {
	RANDOM_TOWNS = true,
	NO_STREAMERS = true,
      },
    },
    actor = {
      class = 'mod.class.generator.actor.T2LevelFilteredActors',
      init_npc = 24,
      respawn_in = 100,
      filters = {
	{ prob=30, mode='any' },
	{ prob=70, mode='or', flags={ 'STUPID', 'WEIRD_MIND', 'SHAPECHANGER', 'ATTR_MULTI', 'CHAR_MULTI', 'EMPTY_MIND', 'INVISIBLE', 'PASS_WALL', 'KILL_WALL', 'MULTIPLY' }, talents={ 'T_BR_CONF', 'T_BR_CHAO', 'T_BA_CHAO', 'T_CONF', 'T_FORGET', 'T_TRAPS' }, random_move=true },
      },
      guardian = 'THE_GLASS_GOLEM',
    },
    object = {
      class = 'mod.class.generator.object.T2LevelFilteredObjects',
      weights = { treasure=50, combat=10, magic=20, tools=20, junk=0 },
    },
    trap = {
      class = 'mod.class.generator.trap.T2LevelFilteredTraps',
    },
  },
  alter_level_data = util.adjust_level,
  levels = {
    [1] = {
      generator = {
	map = {
	  up = 'UP_TO_WILDERNESS',
	  -- up_lev = 1
	},
      },
    },
    [16] = {
      no_level_connectivity = true,
      level_name = [[Machine]],
      enter_desc = [[The clatter of strange machinery surrounds you.]],
      generator = {
	map = {
	  class='mod.class.generator.map.Static',
	  map = 'zones/machine',
	  one_time_level = 'machine',
	  flags = { DESC=true, NO_GENO=true, NO_NEW_MONSTER=true, SPECIAL=true, NO_STAIR=true, ASK_LEAVE=true, NO_TELEPORT=true },
	},
	actor = {
	  init_npc = table.NIL_MERGE,
	  respawn_in = table.NIL_MERGE,
	},
      },
    },
  },
}
