-- $Id: zone.lua 3850 2020-04-05 20:13:34Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2012-2020 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

-- Zone file for Moria.

return {
  name = 'Moria',
  display_abbr = 'MoM',
  level_scheme = 'fixed',
  max_level = 21,
  level_range = { 30, 50 },
  width = 198,
  height = 66,

  generator = {
    map = {
      class = 'mod.class.generator.map.T2DungeonLevel',
      floor = { 'DIRT' },
      fill = { 'MOUNTAIN_CHAIN' },
      outer_wall = 'WALL',
      inner_wall = 'MOUNTAIN_CHAIN',
      fill_step = 0,
      up = 'UP',
      down = 'DOWN',
      flags = {
	WATER_RIVER = true,
	BIG = true,
	NO_STREAMERS = true,
	FORCE_DOWN = true,
	RANDOM_TOWNS = true,
	WILD_45_30__44_37 = true,
      },
    },
    actor = {
      class = 'mod.class.generator.actor.T2LevelFilteredActors',
      init_npc = 40,
      respawn_in = 40,
      filters = {
	{ prob=40, mode='or', flags={ 'ORC' } },
	{ prob=30, mode='or', flags={ 'TROLL', 'GIANT' } },
	{ prob=20, mode='or', flags={ 'DEMON' } },
	{ prob=10, mode='any' },
      },
      guardian = 'DURIN_S_BANE',
    },
    object = {
      class = 'mod.class.generator.object.T2LevelFilteredObjects',
      weights = { treasure=30, combat=50, magic=10, tools=5, junk=5 },
    },
    trap = {
      class = 'mod.class.generator.trap.T2LevelFilteredTraps',
    },
  },
  alter_level_data = util.adjust_level,
  levels = {
    [1] = {
      generator = {
	map = {
	  up = 'UP_TO_WILDERNESS',
	  -- up_lev = 1
	},
      },
    },
    [6] = {
      no_level_connectivity = true,
      level_name = [[Orc Town]],
      enter_desc = [[You hear orc warcries.]],
      generator = {
	map = {
	  class='mod.class.generator.map.Static',
	  map = 'zones/orc-town',
	  one_time_level = 'orc-town',
	  flags = { DESC=true, NO_GENO=true, NO_NEW_MONSTER=true, SPECIAL=true, NO_STAIR=true, ASK_LEAVE=true, NO_TELEPORT=true },
	},
	actor = {
	  init_npc = table.NIL_MERGE,
	  respawn_in = table.NIL_MERGE,
	},
      },
    },
    [11] = {
      generator = {
	map = {
	  down_branch = 'DOWN_TO_SMALL_WATER_CAVE',
	},
      },
    },
    [21] = {
      generator = {
	map = {
	  down = 'DOWN_TO_KHAZAD_DUM',
	},
      },
    },
  },

  entry_point = function(self, lev, from_lev, from_zone)
    if not from_zone then
      return nil
    elseif from_zone.name == 'small-water-cave' then
      -- Returning from a down branch; enter on a down-branch stair.
      return { type='branch', subtype='down' }
    elseif lev == 21 and (util.zone_is(from_zone, 'worldmap') or util.zone_is(from_zone, 'wilderness')) then
      -- Slight Hack(TM):  Coming in the "bottom" entrance of Moria;
      -- start on a down stair.
      return { use='default_down' }
    elseif util.zone_is(from_zone, 'worldmap') or util.zone_is(from_zone, 'wilderness') then
      -- Coming from the world map (or Bree -> Barrow-Downs); start on an
      -- up stair.
      return { use='default_up' }
    else
      return nil
    end
  end,
}
