-- $Id: zone.lua 3272 2018-03-06 04:56:52Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2018 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

local Dialog = require 'engine.ui.Dialog'

return {
  name = "Necromancer's Lair",
  level_scheme = 'fixed',
  max_level = 1,
  level_range = { 45, 45 },
  width = 33,
  height = 23,
  no_level_connectivity = true,
  no_recall_depth = true,

  generator = {
    map = {
      class = 'mod.class.generator.map.Static',
      map = 'quests/minas-anor-haunted',
      flags = { NO_GENO=true },
    },
    actor = {
      class = 'mod.class.generator.actor.T2LevelFilteredActors',
      -- We'll generate actors in post_process() below.
      no_prepop = true,
    },
    object = {
      class = 'mod.class.generator.object.T2LevelFilteredObjects',
      weights = { treasure=20, combat=20, magic=20, tools=20, junk=20 },
      no_prepop = true,	-- This is just for random objects on the map
    },
    trap = {
      class = 'mod.class.generator.trap.T2LevelFilteredTraps',
      -- We'll generate traps in post_process() below.
      n_traps = { 0, 0 },
    },
  },

  -- cf. quest_haunted_gen_hook() in src/q_haunted.c
  post_process = function(level, zone)
    local Map = require 'mod.class.Map'
    local map = game.level.map
    local a_gen = zone:getGenerator('actor', level)
    local t_gen = zone:getGenerator('trap', level)
    -- Place some ghosts.
    local n = 12
    while n > 0 do
      local x, y = rng.range(0, 30) + 1, rng.range(0, 20) + 1
      if map:checkEntity(x, y, Map.TERRAIN, 'floor') and not map:checkEntity(x, y, Map.TERRAIN, 'permanent') then
	local m = a_gen:generateOne{ name='GHOST', sleep=0, x=x, y=y, ret=true }
	if m then
	  m.quest_on_die = 'minas-anor-house'
	  n = n - 1
	end
      end
    end
    -- Place some random monsters to haunt us.
    local cands = {
      'POLTERGEIST', 'GREEN_GLUTTON_GHOST', 'CRYPT_CREEP', 'ROTTING_CORPSE',
      'LOST_SOUL', 'MOANING_SPIRIT', 'FLYING_SKULL', 'GHAST', 'VAMPIRIC_MIST',
      'WHITE_WRAITH', 'GHOUL', 'SHADE', 'SPECTRE', 'HEADLESS_GHOST', 'DREAD',
      'PHANTOM', 'GREY_WRAITH', 'REVENANT', 'CRYPT_THING', 'BLACK_WRAITH',
      'NIGHT_MARE', 'SHADOW',
    }
    n = rng.dice(4, 4)
    while n > 0 do
      local x, y = rng.range(0, 30) + 1, rng.range(0, 20) + 1
      if map:checkEntity(x, y, Map.TERRAIN, 'floor') and not map:checkEntity(x, y, Map.TERRAIN, 'permanent') then
	local m = a_gen:generateOne{ name=rng.table(cands), no_grp=true, sleep=0, x=x, y=y, ret=true }
	if m then
	  m.quest_on_die = 'minas-anor-house'
	  n = n - 1
	end
      end
    end
    -- Place some random traps
    n = rng.dice(4, 4) + 10
    while n > 0 do
      local x, y = rng.range(0, 30) + 1, rng.range(0, 20) + 1
      if map:checkEntity(x, y, Map.TERRAIN, 'floor') and not map:checkEntity(x, y, Map.TERRAIN, 'permanent') then
	t_gen:generateOne{ x=x, y=y }
	n = n - 1
      end
    end
  end,
}
