-- $Id: T2LevelFilteredObjects.lua 3258 2018-03-03 20:14:48Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2012-2018 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

require 'engine.class'
local Map = require 'mod.class.Map'
local ObjRandom = require 'engine.generator.object.Random'
local Object = require 'mod.class.Object'

module(..., package.seeall, class.inherit(ObjRandom))

function _M:init(zone, map, level, flags)
  ObjRandom.init(self, zone, map, level)
  self.data = level.data.generator.object

  flags = flags or {}
  self.force_level = flags.force_level
  if self.force_level and flags.force_avg_level then
    -- Force level to average of zone level and provided level.
    local lev = Object.resolve_flags.force_level or self.zone:level_adjust_level(self.level, 'object')
    self.force_level = math.floor((self.force_level + lev)/2)
  end

  self.no_adj_rating = flags.no_adj_rating
end

function _M:generate()
  if self.data.no_prepop then return end
  -- This roughly duplicates src/generate.c lines 7912-7917.
  local max = rng.normal(9, 3)
  print(('[GENERATOR] placing %d objects in rooms'):format(max))
  for _ = 1, max do self:generateOne{only_room=true} end
  max = rng.normal(3, 3)
  print(('[GENERATOR] placing %d objects everywhere'):format(max))
  for _ = 1, max do self:generateOne() end
  max = rng.normal(3, 3)
  print(('[GENERATOR] placing %d gold everywhere'):format(max))
  for _ = 1, max do self:generateGold() end
end

function _M:getLevel()
  return Object.resolve_flags.force_level or self.force_level or self.zone:level_adjust_level(self.level)
end

function _M:generateOne(t)
  t = t or {}

  local o, x, y
  if t.name then
    o = self.zone:makeEntityByName(self.level, 'object', t.name)
  else
    -- We count on our level's default object filter here if the caller
    -- doesn't provide one.
    o = self.zone:makeEntity(self.level, 'object', t.filter, self:getLevel(), true)
  end
  print('[GENERATOR] created '..(o and o.name or '???'))
  if o then
    x, y = t.x, t.y
    if not t.x or not t.y then
      x, y = rng.range(0, self.map.w-1), rng.range(0, self.map.h-1)
      local tries = 0
      while (not self.map:checkEntity(x, y, Map.TERRAIN, "empty_bold") or
	(t.only_room and not self.map.attrs(x, y, 'room'))) and
	tries < 10000
      do
	x, y = rng.range(0, self.map.w-1), rng.range(0, self.map.h-1)
	tries = tries + 1
      end
      if tries >= 1000 then x, y = nil, nil end
    end
    if x and y then
      print(('[GENERATOR] placing %s at %d,%d'):format(o.name, x, y))
      self.zone:addEntity(self.level, o, "object", x, y)

      local lev = self:getLevel()
      o:setFoundHere(t.found or { type='floor' }, self.zone, self.level)
      o.found.level = lev
      if not o.cursed and self.level.rating and not self.no_adj_rating then
	if o.level > lev then
	  self.level.rating = self.level.rating + o.level - lev
	end
	if o.rating then
	  self.level.rating = self.level.rating + o.rating
	  o.rating = nil
	end
      end
    end
  end
end

function _M:generateGold(t)
  t = t or {}
  local lev = self:getLevel()
  local coin_type = obj_util.choose_coin_type(lev)
  local o = self.zone:makeEntityByName(self.level, 'object', coin_type)
  print('[GENERATOR] created '..(o and o.name or '???'))
  if o then
    x, y = t.x, t.y
    if not t.x or not t.y then
      x, y = rng.range(0, self.map.w-1), rng.range(0, self.map.h-1)
      local tries = 0
      while not self.map:checkEntity(x, y, Map.TERRAIN, "clean_bold") and tries < 10000 do
	x, y = rng.range(0, self.map.w-1), rng.range(0, self.map.h-1)
	tries = tries + 1
      end
      if tries >= 10000 then x, y = nil, nil end
    end
    if x and y then
      print(('[GENERATOR] placing %s at %d,%d'):format(o.name, x, y))
      self.zone:addEntity(self.level, o, "object", x, y)
    end
  end
end
