-- $Id: PlayerLore.lua 3747 2019-10-30 02:27:29Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2019 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

require 'engine.class'
local LorePopup = require 'mod.dialogs.LorePopup'

module(..., package.seeall, class.make)

_M.lore_defs = {}

-- Static
function _M:loadDefinition(file, env)
  env = env or {
    Lore = self,
    newLore = function(t) self:newLore(t) end,
    load = function(f) self:loadDefinition(f, getfenv(2)) end,
  }
  local f, err = loadfile(file)
  if not f and err then error(err) end
  setfenv(f, setmetatable(env, {__index=_G}))
  f()
end

-- Static
function _M:newLore(t)
  assert(t.id, 'no lore ID')
  assert(t.name, 'no lore name')
  assert(t.category, 'no lore category')
  assert(t.lore, 'no lore lore')
  t.order = #self.lore_defs + 1
  self.lore_defs[t.id] = t
  self.lore_defs[#self.lore_defs+1] = t
end

function _M:init(t)
  self.lore_known = self.lore_known or {}
end

function _M:knownLore(id)
  self.lore_known = self.lore_known or {}
  return self.lore_known[id] and true or false
end

function _M:getLore(id, silent)
  self.lore_known = self.lore_known or {}
  if not silent then
    assert(self.lore_defs[id], 'bad lore id '..id)
  end
  return self.lore_defs[id]
end

function _M:learnLore(id, silent)
  local l = self:getLore(id, silent)
  if not l then return end
  local learned = false

  if not self:knownLore(id) or l.always_pop then
    game.logPlayer(self, 'Lore found: #0080FF#%s#LAST#', l.name)
    LorePopup.new(l, game.w * 0.6, 0.8)
    game.logPlayer(self, [[You can read all your collected lore via the game menu (bound to '%s').]], util.getFormattedKeyBinds('EXIT', [[' or ']]))
    learned = true
  end

  self.lore_known[id] = true
  if learned and l.on_learn then l.on_learn(self) end
end
