-- $Id: LorePopup.lua 3747 2019-10-30 02:27:29Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2019 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

require 'engine.class'
local Dialog = require 'engine.ui.Dialog'
local Textzone = require 'engine.ui.Textzone'

module(..., package.seeall, class.inherit(Dialog))

function _M:init(l, w, force_height)
  self.title_shadow = false
  self.color = {r=0x3a, g=0x35, b=0x33}
  self.ui = 'parchment'

  Dialog.init(self, 'Lore found: #0080FF#'..l.name..'#LAST#', 1, 1)

  local lines = l.lore:splitLines(w - 10, self.font)

  local required_h = self.font:height() + self.font_h * (#lines - 1)
  local h = math.min(force_height and (force_height * game.h) or 999999999, required_h)

  self.c_text = Textzone.new {
    width = w + 10,
    height = h,
    scrollbar = (h < required_h and true or false),
    text = l.lore,
    color = self.color,
  }
  self.c_text:setTextShadow(false)

  local function on_end()
    game.tooltip.inhibited = false
    game:unregisterDialog(self)
  end

  self:loadUI {
    { left=3, top=3, ui=self.c_text },
  }
  self.key:addBind("EXIT", on_end)
  self.key:addBind("ACCEPT", on_end)
  self:setupUI(true, true)

  game:registerDialog(self)
  game.tooltip.inhibited = true
end

