-- $Id: ShowStore.lua 3893 2020-04-21 02:31:51Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2012-2020 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

require 'engine.class'
local ShowStore = require 'engine.dialogs.ShowStore'
local StoreItemDialog = require 'mod.dialogs.StoreItemDialog'

module(..., package.seeall, class.inherit(ShowStore))

function _M:init(title, store, store_inven, actor, actor_inven, store_filter, actor_filter, action, desc, descprice, allow_sell, allow_buy, on_select)
  local tooltip_cb = function(item)
    if item.last_display_x and item.object then
      local x
      if self.focus_ui and self.focus_ui.ui == self.c_inven then
	x = item.last_display_x - game.tooltip.w
      elseif self.focus_ui and self.focus_ui.ui == self.c_store then
	x = item.last_display_x + self.c_store.w
      end
      game:tooltipDisplayAtMap(x or item.last_display_x, item.last_display_y, item.object:getDesc({do_color=true}, actor_inven))
    elseif item.last_display_x and item.data and item.data.desc then
      game:tooltipDisplayAtMap(item.last_display_x, item.last_display_y, item.data.desc, {up=true})
    end
    if on_select then on_select(item) end
  end

  self.store = store
  self.actor = actor
  ShowStore.init(self, title, store_inven, actor_inven, store_filter, actor_filter, action, desc, descprice, allow_sell, allow_buy, tooltip_cb)

  -- Widen the 'char' column a bit to stop the @#$%ing annoying wiggling.
  self.c_inven:setColumns {
    {name="", width={26,"fixed"}, display_prop="char", sort="id"},
    {name="", width={24,"fixed"}, display_prop="object", direct_draw=function(item, x, y) item.object:toScreen(nil, x+4, y, 16, 16) end},
    {name="Inventory", width=80, display_prop="name", sort="name"},
    {name="Category", width=20, display_prop="cat", sort="cat"},
    {name="Price", width={50,"fixed"}, display_prop="desc_price", sort=function(a, b) return descprice("sell", a.object) <descprice("sell", b.object) end},
  }
  self.c_store:setColumns {
    {name="", width={26,"fixed"}, display_prop="char", sort="id"},
    {name="", width={24,"fixed"}, display_prop="object", direct_draw=function(item, x, y) item.object:toScreen(nil, x+4, y, 16, 16) end},
    {name="Store", width=80, display_prop="name"},
    {name="Category", width=20, display_prop="cat"},
    {name="Price", width={50,"fixed"}, display_prop="desc_price", sort=function(a, b) return descprice("buy", a.object) <descprice("buy", b.object) end},
  }
end

function _M:needsDialog(item)
  if not item or not item.object then return false end
  if spell.item_spells(item.object, self.actor, true) then return true end
  return false
end

-- Overridden to allow use of StoreItemDialog to browse store spellbooks
-- (and any other similar functionality we might think of later).
function _M:use(item)
  if item and item.object then
    if self.focus_ui and self.focus_ui.ui == self.c_store then
      if self:needsDialog(item) then
	local d = StoreItemDialog.new(false, self.actor, self.store, item.object, item.item, self.store_inven, self)
	game:registerDialog(d)
      elseif util.getval(self.allow_buy, item.object, item.item) then
	self.action('buy', item.object, item.item)
      end
    else
      if util.getval(self.allow_sell, item.object, item.item) then
	self.action('sell', item.object, item.item)
      end
    end
  end
end

function _M:generateList()
  ShowStore.generateList(self)

  -- Slight Hack(TM):  If the generated player or store inventory list is
  -- empty, it won't get properly pushed into the corresponding ListColumns
  -- list, and the 'chars' table we generated will be missing, causing an
  -- error when the user types anything.
  if #self.actor_list == 0 then self.c_inven.list.chars = {} end
  if #self.store_list == 0 then self.c_store.list.chars = {} end
end
